@@ -1,6 +1,6 @@
 /* poppler-form.h: qt interface to poppler
  * Copyright (C) 2007-2008, Pino Toscano <pino@kde.org>
- * Copyright (C) 2008, 2011, 2016, 2017, 2019-2021, Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2008, 2011, 2016, 2017, 2019-2022, Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2012, Adam Reichold <adamreichold@myopera.com>
  * Copyright (C) 2016, Hanno Meyer-Thurow <h.mth@web.de>
  * Copyright (C) 2017, Hans-Ulrich Jüttner <huj@froreich-bioscientia.de>
@@ -12,6 +12,9 @@
  * Copyright (C) 2019, Adrian Johnson <ajohnson@redneon.com>
  * Copyright (C) 2020, Thorsten Behrens <Thorsten.Behrens@CIB.de>
  * Copyright (C) 2020, Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by Technische Universität Dresden
+ * Copyright (C) 2021, Theofilos Intzoglou <int.teo@gmail.com>
+ * Copyright (C) 2023-2025, g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+ * Copyright (C) 2024, Pratham Gandhi <ppg.1382@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +37,7 @@
 #include <functional>
 #include <memory>
 #include <ctime>
+#include <optional>
 #include <QtCore/QDateTime>
 #include <QtCore/QVector>
 #include <QtCore/QList>
@@ -42,6 +46,7 @@
 #include <QtCore/QSharedPointer>
 #include "poppler-export.h"
 #include "poppler-annotation.h"
+#include "poppler-qt6.h"
 
 class Object;
 class Page;
@@ -447,6 +452,14 @@ public:
      */
     bool canBeSpellChecked() const;
 
+    /**
+      Sets the text inside the Appearance Stream to the specified
+      \p text
+
+      \since 24.08
+     */
+    void setAppearanceChoiceText(const QString &text);
+
 private:
     Q_DISABLE_COPY(FormFieldChoice)
 };
@@ -497,6 +510,35 @@ public:
         Organization,
     };
 
+    /**
+     * The type of the certificate
+     *
+     * \since 25.03
+     */
+    enum class CertificateType
+    {
+        X509,
+        PGP
+    };
+
+    /** A signing key can be located in different places
+     sometimes. For the user, it might be easier to pick
+     the key located on a card if it has some visual
+     indicator that it is somehow removable.
+
+     \note a keylocation for a certificate without a private
+     key (cannot be used for signing) will likely be "Unknown"
+
+     \since 23.09
+     */
+    enum class KeyLocation
+    {
+        Unknown, /** We don't know the location */
+        Other, /** We know the location, but it is somehow not covered by this enum */
+        Computer, /** The key is on this computer */
+        HardwareToken /** The key is on a dedicated hardware token, either a smartcard or a dedicated usb token (e.g. gnuk, nitrokey or yubikey) */
+    };
+
     CertificateInfo();
     explicit CertificateInfo(CertificateInfoPrivate *priv);
     ~CertificateInfo();
@@ -568,6 +610,20 @@ public:
      */
     bool isSelfSigned() const;
 
+    /**
+     * Can be used to do qualified electronic signatures (legally binding)
+     *
+     * https://en.wikipedia.org/wiki/Qualified_electronic_signature
+     */
+    bool isQualified() const;
+
+    /**
+     * \return certificate type
+     *
+     * \since 25.03
+     */
+    CertificateType certificateType() const;
+
     /**
       The DER encoded certificate.
      */
@@ -580,6 +636,13 @@ public:
      */
     bool checkPassword(const QString &password) const;
 
+    /**
+     The storage location for this key
+
+     \since 23.09
+     */
+    KeyLocation keyLocation() const;
+
     CertificateInfo(const CertificateInfo &other);
     CertificateInfo &operator=(const CertificateInfo &other);
 
@@ -622,7 +685,8 @@ public:
         CertificateRevoked, ///< The certificate was revoked by the issuing certificate authority.
         CertificateExpired, ///< The signing time is outside the validity bounds of this certificate.
         CertificateGenericError, ///< The certificate could not be verified.
-        CertificateNotVerified ///< The certificate is not yet verified.
+        CertificateNotVerified, ///< The certificate is not yet verified.
+        CertificateVerificationInProgress ///< The certificate is not yet verified but is in progress in the background. See \ref validateAsync \since 24.05
     };
 
     /**
@@ -711,10 +775,32 @@ public:
 
 private:
     Q_DECLARE_PRIVATE(SignatureValidationInfo)
-
+    friend class FormFieldSignature;
     QSharedPointer<SignatureValidationInfoPrivate> d_ptr;
 };
 
+/**
+ * Object help waiting for some async event
+ *
+ * \since 24.05
+ */
+class AsyncObjectPrivate;
+class POPPLER_QT6_EXPORT AsyncObject : public QObject // clazy:exclude=ctor-missing-parent-argument
+{
+    Q_OBJECT
+public:
+    /* Constructor. On purpose not having a QObject parameter
+       It will be returned by shared_ptr or unique_ptr
+    */
+    AsyncObject();
+    ~AsyncObject() override;
+Q_SIGNALS:
+    void done();
+
+private:
+    std::unique_ptr<AsyncObjectPrivate> d;
+};
+
 /**
   A form field that represents a signature.
  */
@@ -729,7 +815,9 @@ public:
         UnknownSignatureType,
         AdbePkcs7sha1,
         AdbePkcs7detached,
-        EtsiCAdESdetached
+        EtsiCAdESdetached,
+        UnsignedSignature, ///< \since 22.02
+        G10cPgpSignatureDetached, ///< nonstandardized signature type \since 25.01
     };
 
     /**
@@ -739,6 +827,8 @@ public:
     {
         ValidateVerifyCertificate = 1, ///< Validate the certificate.
         ValidateForceRevalidation = 2, ///< Force revalidation of the certificate.
+        ValidateWithoutOCSPRevocationCheck = 4, ///< Do not contact OCSP servers to check for certificate revocation status \since 21.10
+        ValidateUseAIACertFetch = 8 ///< Use the AIA extension for certificate fetching \since 21.10
     };
 
     /// \cond PRIVATE
@@ -757,26 +847,148 @@ public:
       Validate the signature with now as validation time.
 
       Reset signature validatation info of scoped instance.
+
+      \note depending on the backend, some options are only
+      partially respected. In case of the NSS backend, the two options
+      requiring network access, AIAFetch and OCSP,
+      can be toggled individually. In case of the GPG backend, if either
+      OCSP is used or AIAFetch is used, the other one is also used.
+
+      \deprecated Please rewrite to the async version, that allows the network traffic part of fetching to happen in the background
      */
-    SignatureValidationInfo validate(ValidateOptions opt) const;
+    POPPLER_QT6_DEPRECATED SignatureValidationInfo validate(ValidateOptions opt) const;
 
     /**
       Validate the signature with @p validationTime as validation time.
 
       Reset signature validatation info of scoped instance.
+
+      \note depending on the backend, some options are only
+      partially respected. In case of the NSS backend, the two options
+      requiring network access, AIAFetch and OCSP,
+      can be toggled individually. In case of the GPG backend, if either
+      OCSP is used or AIAFetch is used, the other one is also used.
+
+      \deprecated Please rewrite to the async version, that allows the network traffic part of fetching to happen in the background
      */
-    SignatureValidationInfo validate(int opt, const QDateTime &validationTime) const;
+    POPPLER_QT6_DEPRECATED SignatureValidationInfo validate(int opt, const QDateTime &validationTime) const;
+
+    /**
+      Validate the signature with @p validationTime as validation time.
+
+      Reset signature validatation info of scoped instance.
+
+      \since 24.05
+
+      \note depending on the backend, some options are only
+      partially respected. In case of the NSS backend, the two options
+      requiring network access, AIAFetch and OCSP,
+      can be toggled individually. In case of the GPG backend, if either
+      OCSP is used or AIAFetch is used, the other one is also used.
+
+      \note certificate validation will have started when this function return. See \ref validateResult on how to get certifcate validation
+      \note connections to \ref AsyncObject must happen by the caller
+      before returning control to the event loop, else signals is not guaranteed to be delivered
+    */
+    std::pair<SignatureValidationInfo, std::shared_ptr<AsyncObject>> validateAsync(ValidateOptions opt, const QDateTime &validationTime = {}) const;
+
+    /**
+     * \return the updated signature validation info from validateAsync
+     * \note that this function will block if the result is not yet ready.
+     * Wait for the \ref AsyncObject::done signal to avoid this function blocking on an inconvenient time
+     *
+     * \since 24.05
+     */
+    SignatureValidationInfo::CertificateStatus validateResult() const;
+
+    /**
+     * For consumers, this enum will be extended. Unknown numbers should probably be
+     * treated as GenericSigningError
+     *
+     * \since 22.02
+     */
+    enum SigningResult
+    {
+        FieldAlreadySigned, ///< Trying to sign a field that is already signed \since 24.10
+        GenericSigningError, ///< Unclassified error \since 24.10
+        SigningSuccess, ///< No error \since 24.10
+        InternalError, ///< Unexpected error, likely a bug in poppler \since 24.12
+        KeyMissing, ///< Key not found (Either the input key is not from the list or the available keys has changed underneath \since 24.12)
+        WriteFailed, ///< Write failed (permissions, faulty disk, ...) \since 24.12
+        UserCancelled, ///< User cancelled the process \since 24.12
+        BadPassphrase, ///< Passphrase didn't work \since 25.03
+    };
+
+    /**
+      Signs a field of UnsignedSignature type.
+
+      Ignores data.page(), data.fieldPartialName() and data.boundingRectangle()
+
+      \since 22.02
+     */
+    SigningResult sign(const QString &outputFileName, const PDFConverter::NewSignatureData &data) const;
 
 private:
     Q_DISABLE_COPY(FormFieldSignature)
 };
+/**
+ * Possible compiled in backends for signature handling
+ *
+ * \since 23.06
+ */
+enum class CryptoSignBackend
+{
+    NSS,
+    GPG
+};
+
+/**
+ * The available compiled-in backends
+ *
+ * \since 23.06
+ */
+QVector<CryptoSignBackend> POPPLER_QT6_EXPORT availableCryptoSignBackends();
+
+/**
+ * Returns current active backend or nullopt if none is active
+ *
+ * \note there will always be an active backend if there is available backends
+ *
+ * \since 23.06
+ */
+std::optional<CryptoSignBackend> POPPLER_QT6_EXPORT activeCryptoSignBackend();
+
+/**
+ * Sets active backend
+ *
+ * \return true on success
+ *
+ * \since 23.06
+ */
+bool POPPLER_QT6_EXPORT setActiveCryptoSignBackend(CryptoSignBackend backend);
+
+enum class CryptoSignBackendFeature
+{
+    /// If the backend itself out of band requests passwords
+    /// or if the host applicaion somehow must do it
+    BackendAsksPassphrase
+};
+
+/**
+ * Queries if a backend supports or not supports a given feature.
+ *
+ * \since 23.06
+ */
+bool POPPLER_QT6_EXPORT hasCryptoSignBackendFeature(CryptoSignBackend, CryptoSignBackendFeature);
 
 /**
   Returns is poppler was compiled with NSS support
 
+  \deprecated Use availableBackends instead
+
   \since 21.01
 */
-bool POPPLER_QT6_EXPORT hasNSSSupport();
+bool POPPLER_QT6_DEPRECATED POPPLER_QT6_EXPORT hasNSSSupport();
 
 /**
   Return vector of suitable signing certificates
@@ -805,6 +1017,23 @@ void POPPLER_QT6_EXPORT setNSSDir(const QString &pathURL);
   \since 21.01
 */
 void POPPLER_QT6_EXPORT setNSSPasswordCallback(const std::function<char *(const char *)> &f);
+
+/**
+ * Allow pgp signatures in pdf files (standard-extension, experimental)
+ * Not supported for NSS backend
+ *
+ * \param allowed new value for pgp signatures allowed
+ *
+ * \since 25.03
+ */
+void POPPLER_QT6_EXPORT setPgpSignaturesAllowed(bool allowed);
+
+/**
+ * \returns if pgp signatures are allowed.
+ *
+ * \since 25.03
+ */
+bool POPPLER_QT6_EXPORT arePgpSignaturesAllowed();
 }
 
 #endif
